% Kim and Le (2017): Grouped Hetrogeneity in Pennel Data Models
%
% y_{it} = x_{it}*gamma + beta_{it} + e_{it}, e_{it} ~ t(0,sigma^2)
% for i = 1,2,...,N, t =1,2,...,T
%
% Jaeho Kim 
% University of Oklahoma
% 2017_11_19

clear all;
clc;
close all;
rng(4561);



%% Load Data
load('final_data');

[coutiries_id_num,coutiries_id_txt,coutiries_id_raw] = xlsread('country_id.xls');

Z = ldem_linc;
Y = dem;
W = ones(length(Z),1);
X = Z;

T = 7;
N = 90;
K = 2;

% Contruct data matrix
Y_i = reshape(Y,T,1,N);
X_i = zeros(T,2,N);
X_i(:,1,:) = reshape(X(:,1),T,N);
X_i(:,2,:) = reshape(X(:,2),T,N);
conid_i = reshape(coutiries_id_txt,T,N);
conid_i = conid_i(1,:)';

% Compute cross sectional average
bar_X = mean(X_i(:,2,:),3);

W_i = zeros(T,T,N);
I_i = zeros(T,T,N);
W0_i = kron(ones(1,N),bar_X);
I0_i = reshape(W,T,N);
W_cat = [];
I_cat = [];

Idt_mat = eye(T);
for i = 1:N
    Wt_i = W0_i(:,i);
    It_i = I0_i(:,i);
    for t = 1:T
        W_i(t,:,i) = kron(Idt_mat(t,:),Wt_i(t,:));
        I_i(t,:,i) = kron(Idt_mat(t,:),It_i(t,:));
    end
    W_cat = [W_cat;W_i(:,:,i)];
    I_cat = [I_cat;I_i(:,:,i)];
end
taxis = (1970:5:2000)';





%% MCMC Estimation (saving space and priors)
[T,KW,N] = size(W_i);
TN = T*N;

numMCMC = 50000;          % Number of iterations in the MCMC samplers
burn_in = 50000;          % Number of interations to burn

% OLS Estimates
Y_ols = Y;
X_ols = [W_cat X];
ols_est = inv(X_ols'*X_ols)*X_ols'*Y_ols;
sig2_est = (Y_ols - X_ols*ols_est)'*(Y_ols - X_ols*ols_est)/TN;
ols_se = sig2_est*inv(X_ols'*X_ols);


% Prior Specification
% gamma (common parameters)
gamma_bpr = zeros(2,1);
gamma_spr = (10^2)*eye(2);

% theta (group-specific parameters)
theta_bpr = ols_est(1:T,1);
theta_spr = 30*ols_se(1:7,1:7);

% Q_g
Qpr = 4*[1;(0.01)^2];

% sigma2
sig2pr = 6*[1;(0.1)^2];

% alpha
alpha_pr = [0.4;10];                   
                       
% nu
nu_pr = 25;


% % % Check prior for gamma distribution
% alpha_pr_test = gamrnd(alpha_pr(1,1),alpha_pr(2,1),100000,1);
% hist(alpha_pr_test,100)
% a = shape, b = scale
% E[x] = a*b, V[x] = a*b^2

% Note: when alpha is large, more likeliy to have a new cluster
%       when alpha is small, less likeliy to have a new cluster

% Saving Space (common parameters)
gamma_mat = zeros(numMCMC,2);
sig2_mat = zeros(numMCMC,1);
G_mat = zeros(numMCMC,N);
alpha_mat = zeros(numMCMC,1);
corr_mat = zeros(numMCMC,2);

% Saving Space (group parameters)
clstn_mat = zeros(numMCMC,1);
beta1i_mat = zeros(numMCMC,N);
beta2i_mat = zeros(numMCMC,N);
beta3i_mat = zeros(numMCMC,N);
beta4i_mat = zeros(numMCMC,N);
beta5i_mat = zeros(numMCMC,N);
beta6i_mat = zeros(numMCMC,N);
beta7i_mat = zeros(numMCMC,N);
beta_mat = [];

% MCMC mean of log-liklihoods
lik_mat = zeros(T*N,2);





%% MCMC Estimation (Initial values)
% Initial Values based on pooled OLS
gamma_draw = ols_est(8:9,1);
sig2_draw = sig2_est;
alpha_draw = 1;


% Generate candidate grouped heterogeneity from OLS estimates
Cmax = 5000;
thetam_temp = (kron(ones(1,Cmax),ols_est(1:7,1)) + chol(theta_spr)'*randn(7,Cmax))';
betam_temp = thetam_temp.*kron(ones(Cmax,1),bar_X');


% Draw group indicator variable
G_draw = ones(N,1);

for i = randperm(N)
    
     % Step 1: Select observations
     I_indv = I_i(:,:,i);
     X_indv = X_i(:,:,i);
     Y_indv = Y_i(:,:,i);
     
     % Step 2: Compute density
     Ydev = Y_indv - X_indv*gamma_draw;
     lny_den = zeros(Cmax,1);
     indy = zeros(Cmax,1);
     for c = 1:Cmax
         lny_den(c,1) = -0.5*T*log(2*pi*sig2_draw) ...
             - 0.5*(Ydev - I_indv*betam_temp(c,:)')'*(Ydev - I_indv*betam_temp(c,:)')/sig2_draw;
     end       
     y_den = exp(lny_den - max(lny_den));
     G_draw(i,1) = discreteRnd(y_den);
     
 end;
 
% Adjust cluster index 
[G_unique,idx_a,idx_c]  = unique(G_draw);
betam_draw = betam_temp(G_unique,:);
G_draw = idx_c;
index_mat = kron(G_draw,ones(T,1));
beta_draw = betam_draw(index_mat,:);
Cmax = length(G_unique) + 100;

sig2_draw = 0.5*sig2_est;
Qm_draw = 0.5*sig2_est*ones(size(betam_draw,1),1);


%% Run MCMC loop
tic;
reverseStr = [];
for itr = 1:(numMCMC+burn_in)
    reverseStr = displayprogress(100*itr/(numMCMC+burn_in), reverseStr);
     

    
    
    %% Step 1: Draw V and U  (Update active components)
    % Draw V
    % count elements in clusters 
    Cact = max(G_draw);
    Nd = zeros(Cact,1);
    for c = 1:Cact
        Nd(c,1) = length( find(G_draw == c) );
    end

    % update v_1
    v_pst_a = Nd(1,1) + 1;
    v_pst_b = sum(Nd(2:end,1)) + alpha_draw;
    v1_draw = betarnd(v_pst_a,v_pst_b);

    % update v_c's upto c = Cmax and compuate weights pi_c
    % c = 1
    V_draw = zeros(Cact,1);
    pi_draw = zeros(Cact,1);
    V_draw(1,1) = v1_draw;
    pi_draw(1,1) = v1_draw;
    % c = 2,3,...,Cact-1
    for c = 2:(Cact-1)
        v_pst_a = Nd(c,1) + 1;
        v_pst_b = sum(Nd(c+1:end)) + alpha_draw;
        V_draw(c,1) = betarnd(v_pst_a,v_pst_b);
        pi_draw(c,1) = V_draw(c,1)*prod(1-V_draw(1:c-1,1));
    end
    % c = Cact
    v_pst_a = Nd(Cact,1) + 1;
    v_pst_b = alpha_draw;
    V_draw(Cact,1) = betarnd(v_pst_a,v_pst_b);
    pi_draw(Cact,1) = V_draw(Cact,1)*prod(1-V_draw(1:Cact-1,1));    
    
    
    
    
    
    %% Step 2: draw cluster-dependent parameters (only for active beta_c) 
    % For active clusters
    thetam_draw = zeros(Cact,KW);
    betam_draw = zeros(Cact,KW);
    Qm_temp = zeros(Cact,1);

    for c = 1:Cact
        loc = find(G_draw == c);
        n_c = size(loc,1);
        if n_c > 0;
            % Find indepedent and depedent observations in a cluster
            I_ic = I_i(:,:,loc);
            W_ic = W_i(:,:,loc);
            Y_ic = Y_i(:,:,loc);
            X_ic = X_i(:,:,loc);

            % Redefine data matrix   
            Y_c = [];
            W_c = [];
            X_c = [];
            I_c = [];
            for i = 1:n_c
                Y_c = [Y_c;Y_ic(:,:,i)];
                W_c = [W_c;W_ic(:,:,i)];
                I_c = [I_c;I_ic(:,:,i)];
                X_c = [X_c;X_ic(:,:,i)];
            end
            
            Y_thetac = Y_c - X_c*gamma_draw;
            X_thetac = W_c;
            Y_betac = Y_thetac;
            X_betac = I_c;
            M_g = sig2_draw*eye(n_c*T) + kron(ones(n_c,1)*ones(1,n_c),Qm_draw(c,1)*eye(T));
        else
            Y_thetac = 0;
            X_thetac = 0; 
            Y_betac = 0;
            X_betac = 0;
            M_g = 1;
        end
        
        % sample theta_m, prior mean of beta_m (partially collapsed Gibbs)
        [thetac_draw,b_pst,s_pst] = normal_bayes_reg(Y_thetac,X_thetac,M_g,theta_bpr,theta_spr); 
        thetam_draw(c,:) = thetac_draw';
        
        % sample beta_m
        beta_pr_mean = W_i(:,:,1)*thetac_draw;
        beta_pr_var = Qm_draw(c,1)*eye(T);
        
        [betac_draw,b_pst,s_pst] = normal_bayes_reg(Y_betac,X_betac,sig2_draw,beta_pr_mean,beta_pr_var); 
        betam_draw(c,:) = betac_draw';
        
        % sample Q_m
        er_b = betac_draw - beta_pr_mean;
        Qvd_pst = (T + Qpr(1,1))/2; 
        Qnu_pst = (er_b'*er_b + Qpr(2,1))/2;         
        Qc_draw = 1/(gamrnd(Qvd_pst,1/Qnu_pst));
        Qm_temp(c,1) = Qc_draw;
        
    end  
    Qm_draw = Qm_temp;
    
    
    
    % label switching moves for active set
	if (rand(1) < 0.5) 
        
		% (A) randomly choose two components and propose to swap
		j = randsample(1:Cact, 2, 0);
		MHaccept = ( Nd(j(1))-Nd(j(2)) )* ( log(pi_draw(j(2))) - log(pi_draw(j(1))) );
        
		if log(rand(1)) < min(0,(MHaccept))
			tmp = G_draw;
			G_draw(find(tmp==j(1))) = j(2);
			G_draw(find(tmp==j(2))) = j(1);
            
			tmp = betam_draw;
			betam_draw(j(1),:) = tmp(j(2),:);
			betam_draw(j(2),:) = tmp(j(1),:);
            
            tmp = thetam_draw;
            thetam_draw(j(1),:) = tmp(j(2),:);
			thetam_draw(j(2),:) = tmp(j(1),:);
            
            tmp = Qm_draw;
            Qm_draw(j(1),:) = tmp(j(2),:);
			Qm_draw(j(2),:) = tmp(j(1),:);            
            
        end
        
    else
        
		if (Cact > 2)
            
			% (B) choose a neighbouring pair to switch
			if Cact == Cmax
                if Cact == 3
                    j = 2;
                else
                    j = randsample(2:Cact-1, 1, 1, Nd(2:Cact-1));
                end
			else
				j = randsample(2:Cact, 1, 1, Nd(2:Cact));
			end

			MHaccept = Nd(j-1)*log(1-V_draw(j)) - Nd(j)*log(1-V_draw(j-1));
			if log(rand(1)) < min(0, (MHaccept))

				tmp = V_draw;
				V_draw(j) = tmp(j-1);
				V_draw(j-1) = tmp(j);

				pi_draw(:) = 0;
				for c = 1:Cact
					pi_draw(c,1) = V_draw(c,1)*prod(1-V_draw(1:c-1,1));
                end
         
				tmp = G_draw;
				G_draw(find(tmp==j)) = j-1;
				G_draw(find(tmp==j-1)) = j;

				tmp = betam_draw;
				betam_draw(j,:) = tmp(j-1,:);
				betam_draw(j-1,:) = tmp(j,:);

                tmp = thetam_draw;
				thetam_draw(j,:) = tmp(j-1,:);
				thetam_draw(j-1,:) = tmp(j,:);
                
                tmp = Qm_draw;
				Qm_draw(j,:) = tmp(j-1,:);
				Qm_draw(j-1,:) = tmp(j,:);
                
            end
        end
    end   
    Cact = max(G_draw);    
    Nd = zeros(Cact,1);
    % update the number of components in each cluster
    for c = 1:Cact
        Nd(c,1) = length( find(G_draw == c) );
    end       
    

    %% Step 3: draw U conditional on V
    U_draw = rand(N,1).*pi_draw(G_draw);    
 
    
    
    
    %% Step4: Draw alpha
    % simulate eta from Beta(alpha+1,n)
    eta_draw = betarnd(alpha_draw+1,N);
    
	% simulate alpha given eta
	uz = length(unique(G_draw));
	temp_wg = (uz + alpha_pr(1,1) -1)/(N*(alpha_pr(2,1)-log(eta_draw)));
	wg = temp_wg/(1+temp_wg);
	% wg is the mixture weight in the mixture of Gamma's
	alpha_draw = gamrnd(alpha_pr(1,1)+uz-randsample(2, 1, 1, [wg; 1-wg])+1, 1/(alpha_pr(2,1)-log(eta_draw)));    
          
    
    
    
    
    
    
    %% Step 5: Compute new Cmax and generate cluster-dependent parameters for inactive clusters 
    U_min = min(U_draw);
    pi_sum = sum(pi_draw);
    
    if pi_sum > (1 - U_min)
        pi_csum = cumsum(pi_draw);
        Cmax = min(find(pi_csum > (1 - U_min)));        
    else
        % sample more v's until the pi's satisfy \sum_{j=1}^{k*} pi_j > 1-min(u)
        Cmax = Cact;
        while pi_sum < (1 - U_min) 
            Cmax = Cmax + 1;
            vn_draw = betarnd(1,alpha_draw);
            pin_draw = vn_draw*prod(1-V_draw);
            V_draw = [V_draw;vn_draw];
            pi_draw = [pi_draw;pin_draw];
            pi_sum = pi_sum + pin_draw;
            
            
            % sample theta_m from prior
            Y_thetac = 0;
            X_thetac = 0; 
            [thetac_draw,b_pst,s_pst] = normal_bayes_reg(Y_thetac,X_thetac,1,theta_bpr,theta_spr); 
            thetam_draw = [thetam_draw;thetac_draw'];            
            
            % sample Q_m from prior
            Qvd_pst = Qpr(1,1)/2; 
            Qnu_pst = Qpr(2,1)/2;         
            Qc_draw = 1/(gamrnd(Qvd_pst,1/Qnu_pst));
            Qm_draw = [Qm_draw;Qc_draw];
      
            % sample beta_m from prior
            beta_pr_mean = W_i(:,:,1)*thetac_draw;
            beta_pr_var = Qc_draw*eye(T);
            Y_betac = 0;
            X_betac = 0; 
            [betac_draw,b_pst,s_pst] = normal_bayes_reg(Y_betac,X_betac,sig2_draw,beta_pr_mean,beta_pr_var); 
            betam_draw = [betam_draw;betac_draw'];
        end
    end   
    % Update for the next iteration
    index_mat = kron(G_draw,ones(T,1));
    beta_draw = betam_draw(index_mat,:);
        
 
    
    %% Step 6: draw cluster-free parameters (gamma,sigma^2) 
    % draw gamma
    Y_gamma = (Y - sum(I_cat.*beta_draw,2));
    X_gamma = X; 
    [gamma_draw,b_pst,s_pst] = normal_bayes_reg(Y_gamma,X_gamma,sig2_draw,gamma_bpr,gamma_spr); 

    % StepI.2: draw sig2e    
    e_draw = (Y_gamma - X_gamma*gamma_draw);
    sig2d_pst = (TN + sig2pr(1,1))/2; 
    sig2nu_pst = (e_draw'*e_draw + sig2pr(2,1))/2;  
    sig2_draw = 1/(gamrnd(sig2d_pst,1/sig2nu_pst));      
    
    

    %% Step 7: draw group indicator variables
    for i = randperm(N)
        %i = 1
        % Step 1.1: Select observations
        W_indv = W_i(:,:,i);
        I_indv = I_i(:,:,i);
        X_indv = X_i(:,:,i);
        Y_indv = Y_i(:,:,i);
        U_indv = U_draw(i,1);
        
        % Step 1.2: Remove empty cluster
        Ydev = Y_indv - X_indv*gamma_draw;
        lny_den = zeros(Cmax,1);
        indy = zeros(Cmax,1);
        for c = 1:Cmax
            lny_den(c,1) = -0.5*T*log(2*pi*sig2_draw) ...
                - 0.5*(Ydev - I_indv*betam_draw(c,:)')'*(Ydev - I_indv*betam_draw(c,:)')/sig2_draw;
            indy(c,1) = (U_indv < pi_draw(c,1));
        end       
        y_den = exp(lny_den - max(lny_den)).*indy;
        G_draw(i,1) = discreteRnd(y_den);
    end;
    % Update for the next iteration
    index_mat = kron(G_draw,ones(T,1));
    beta_draw = betam_draw(index_mat,:);
  
    
    %% Step 8: store MCMC samples
    if itr > burn_in
 
        % compute correlation
        beta_corr = betam_draw(G_draw,:);
        corr_temp1 = 0;
        corr_temp2 = 0;
        for q = 1:N
           corr_temp1 = corr_temp1 + corrcoef(X_i(:,2,q),beta_corr(q,:)')/N;
        end
        for t = 1:T
           corr_temp2 = corr_temp2 + corrcoef(X_i(t,2,:),beta_corr(:,t)')/T;
        end        
        corr_mat(itr-burn_in,1) = corr_temp1(1,2);
        corr_mat(itr-burn_in,2) = corr_temp2(1,2);
        
        % common parameters
        gamma_mat(itr-burn_in,:) = gamma_draw';
        sig2_mat(itr-burn_in,:) = sig2_draw;
        G_mat(itr-burn_in,:) = G_draw';          
        alpha_mat(itr-burn_in,:) = alpha_draw;   
 
                
        % Save beta for individual countries
        beta_save = betam_draw(G_draw,:);
        beta1i_mat(itr-burn_in,:) = beta_save(:,1)';
        beta2i_mat(itr-burn_in,:) = beta_save(:,2)';
        beta3i_mat(itr-burn_in,:) = beta_save(:,3)';
        beta4i_mat(itr-burn_in,:) = beta_save(:,4)';
        beta5i_mat(itr-burn_in,:) = beta_save(:,5)';
        beta6i_mat(itr-burn_in,:) = beta_save(:,6)';
        beta7i_mat(itr-burn_in,:) = beta_save(:,7)';
        
        % Save unique beta 
        beta_unique = unique(beta_draw,'rows');
        clstn_mat(itr-burn_in,:) = size(beta_unique,1);  
       
        % compute likelihood for model comparison
        Y_gamma = (Y - sum(I_cat.*beta_draw,2));
        X_gamma = X; 
        e_draw = (Y_gamma - X_gamma*gamma_draw);      
        lik = (exp(-0.5*(e_draw.^2)/sig2_draw)/sqrt(2*pi*sig2_draw));
        lik_mat(:,1) = lik_mat(:,1) + (1/numMCMC)*lik;
        lik_mat(:,2) = lik_mat(:,2) + (1/numMCMC)*log(lik);        
        
        
    end

end
toc;
stt_itr = 1;
end_itr = numMCMC;



%% Export Fixed Effects
filename = 'lik_mean.xlsx';
xlswrite(filename,lik_mat)




