% Kim and Le (2017): Grouped Hetrogeneity in Pennel Data Models
%
% y_{it} = x_{it}*gamma + beta_{it} + e_{it}, e_{it} ~ N(0,sigma_i^2)
% for i = 1,2,...,N, t =1,2,...,T
%
% Jaeho Kim 
% University of Oklahoma
% 2017_11_19


clear all;
clc;
close all;
rng(8187);





%% Load Data
load('final_data');
[lik_mean,lik_mean_txt,lik_mean_raw] = xlsread('lik_mean.xlsx');
[coutiries_id_num,coutiries_id_txt,coutiries_id_raw] = xlsread('country_id.xls');

Z = ldem_linc;
Y = dem;
W = ones(length(Z),1);
X = Z;

T = 7;
N = 90;
K = 2;

% Contruct data matrix
Y_i = reshape(Y,T,1,N);
X_i = zeros(T,2,N);
X_i(:,1,:) = reshape(X(:,1),T,N);
X_i(:,2,:) = reshape(X(:,2),T,N);
conid_i = reshape(coutiries_id_txt,T,N);
conid_i = conid_i(1,:)';

W_i = zeros(T,T,N);
W0_i = reshape(W(:,1),T,N);
W_cat = [];

Idt_mat = eye(T);
for i = 1:N
    Wt_i = W0_i(:,i);
    for t = 1:T
        W_i(t,:,i) = kron(Idt_mat(t,:),Wt_i(t,:));
    end
    W_cat = [W_cat;W_i(:,:,i)];
end
taxis = (1970:5:2000)';






%% MCMC Estimation (saving space and priors)
[T,KW,N] = size(W_i);
TN = T*N;

numMCMC = 50000;         % Number of iterations in the MCMC samplers
burn_in = 50000;          % Number of interations to burn

% OLS Estimates
Y_ols = Y;
X_ols = [W_cat X];
ols_est = inv(X_ols'*X_ols)*X_ols'*Y_ols;
sig2_est = (Y_ols - X_ols*ols_est)'*(Y_ols - X_ols*ols_est)/TN;
ols_se = sig2_est*inv(X_ols'*X_ols);


% Prior Specification
% gamma
gamma_bpr = zeros(2,1);
gamma_spr = (10^2)*eye(2);

% beta
beta_bpr = ols_est(1:T,1);
beta_spr = 30*ols_se(1:7,1:7);


% sigma2_v
sig2vpr = 6*[1;(0.1)^2];

% Check prior for inverse gamma
% sig2vd_pst = (sig2vpr(1,1))/2; 
% sig2vnu_pst = (sig2vpr(2,1))/2;  
% sv2_draw = 1./(gamrnd(sig2vd_pst,1/sig2vnu_pst,100000,1));
% hist(sv2_draw,100)
% a = shape, b = scale
% E[x] = a*b, V[x] = a*b^2

% alpha
alpha_pr = [0.4;10];                  
   

% convergence check:
converge_num = 5;

alpha_cv = zeros(numMCMC,1,converge_num);
gnum_cv = zeros(numMCMC,1,converge_num);
gamma_cv = zeros(numMCMC,2,converge_num);
inig_cv = zeros(1,converge_num);


conv_iter = 1;
while conv_iter < converge_num + 1;

% Check prior for gamma
% alpha_pr_test = gamrnd(alpha_pr(1,1),alpha_pr(2,1),100000,1);
% hist(alpha_pr_test,100)
% a = shape, b = scale
% E[x] = a*b, V[x] = a*b^2

% Note: when alpha is large, more likeliy to have a new cluster
%       when alpha is small, less likeliy to have a new cluster


% Saving Space
gamma_mat = zeros(numMCMC,2);
% sig2i_mat = zeros(numMCMC,N);
G_mat = zeros(numMCMC,N);
alpha_mat = zeros(numMCMC,1);


clstn_mat = zeros(numMCMC,1);
beta1i_mat = zeros(numMCMC,N);
beta2i_mat = zeros(numMCMC,N);
beta3i_mat = zeros(numMCMC,N);
beta4i_mat = zeros(numMCMC,N);
beta5i_mat = zeros(numMCMC,N);
beta6i_mat = zeros(numMCMC,N);
beta7i_mat = zeros(numMCMC,N);
beta_mat = [];


% MCMC variance of log-liklihoods
lik_var = zeros(T*N,1);








%% MCMC Estimation (Initial values)
% Initial Values based on pooled OLS
gamma_draw = ols_est(8:9,1);
sig2_draw = sig2_est;
alpha_draw = 1;



% Draw group indicator variable
if conv_iter == 1;
    betam_temp = ols_est(1:7,1)';
    G_draw = ones(N,1);
else
    % Generate candidate grouped heterogeneity from OLS estimates
    Cmax = 5000;
    betam_temp = (kron(ones(1,Cmax),ols_est(1:7,1)) + chol(beta_spr)'*randn(7,Cmax))';

    G_draw = ones(N,1);
    for i = randperm(N)

         % Step 1: Select observations
         W_indv = W_i(:,:,i);
         X_indv = X_i(:,:,i);
         Y_indv = Y_i(:,:,i);

         % Step 2: Compute density
         Ydev = Y_indv - X_indv*gamma_draw;
         lny_den = zeros(Cmax,1);
         indy = zeros(Cmax,1);
         for c = 1:Cmax
             lny_den(c,1) = -0.5*T*log(2*pi*sig2_draw) ...
                 - 0.5*(Ydev - W_indv*betam_temp(c,:)')'*(Ydev - W_indv*betam_temp(c,:)')/sig2_draw;
         end       
         y_den = exp(lny_den - max(lny_den));
         G_draw(i,1) = discreteRnd(y_den);

     end;
end

 
% Adjust cluster index 
[G_unique,idx_a,idx_c]  = unique(G_draw);
betam_draw = betam_temp(G_unique,:);
sig2m_draw = sig2_draw*ones(length(G_unique),T);
G_draw = idx_c;
index_mat = kron(G_draw,ones(T,1));
beta_draw = betam_draw(index_mat,:);
Cmax = length(G_unique) + 100;

inig_cv(1,conv_iter) = size(G_unique,1);





%% Run MCMC loop
ac_num = 0; %ratio of acceptance for nu
tic;
reverseStr = [];
for itr = 1:(numMCMC+burn_in)
    reverseStr = displayprogress(100*itr/(numMCMC+burn_in), reverseStr);
     

    
    %% Step 1: Draw V and U  (Update active components)
    % Draw V
    % count elements in clusters 
    Cact = max(G_draw);
    Nd = zeros(Cact,1);
    for c = 1:Cact
        Nd(c,1) = length( find(G_draw == c) );
    end

    % update v_1
    v_pst_a = Nd(1,1) + 1;
    v_pst_b = sum(Nd(2:end,1)) + alpha_draw;
    v1_draw = betarnd(v_pst_a,v_pst_b);

    % update v_c's upto c = Cmax and compuate weights pi_c
    % c = 1
    V_draw = zeros(Cact,1);
    pi_draw = zeros(Cact,1);
    V_draw(1,1) = v1_draw;
    pi_draw(1,1) = v1_draw;
    % c = 2,3,...,Cact-1
    for c = 2:(Cact-1)
        v_pst_a = Nd(c,1) + 1;
        v_pst_b = sum(Nd(c+1:end)) + alpha_draw;
        V_draw(c,1) = betarnd(v_pst_a,v_pst_b);
        pi_draw(c,1) = V_draw(c,1)*prod(1-V_draw(1:c-1,1));
    end
    % c = Cact
    v_pst_a = Nd(Cact,1) + 1;
    v_pst_b = alpha_draw;
    V_draw(Cact,1) = betarnd(v_pst_a,v_pst_b);
    pi_draw(Cact,1) = V_draw(Cact,1)*prod(1-V_draw(1:Cact-1,1));    
    
    
    
    
    
    
    %% Step 2: draw cluster-dependent parameters (only for active beta_c and sig2_c) 
    % For active clusters
    betam_draw = zeros(Cact,KW);
    sig2m_temp = zeros(Cact,T);
    for c = 1:Cact
        loc = find(G_draw == c);
        n_c = size(loc,1);
        if n_c > 0;
            % Find indepedent and depedent observations in a cluster
            W_ic = W_i(:,:,loc);
            Y_ic = Y_i(:,:,loc);
            X_ic = X_i(:,:,loc);
            sig2c = sig2m_draw(c,:)';
        
            % Redefine data matrix   
            Y_c = [];
            W_c = [];
            X_c = [];
            for i = 1:length(loc)
                Y_c = [Y_c;Y_ic(:,:,i)./sqrt(sig2c)];
                W_c = [W_c;diag(diag(W_ic(:,:,i))./sqrt(sig2c))];
                X_c = [X_c;X_ic(:,:,i)./sqrt([sig2c sig2c])];
            end
            
            Y_betac = Y_c - X_c*gamma_draw;
            X_betac = W_c;
            [betac_draw,b_pst,s_pst] = normal_bayes_reg(Y_betac,X_betac,1,beta_bpr,beta_spr); 
            betam_draw(c,:) = betac_draw';            
            
            % draw sig2v_m
            for t = 1:T
                Y_c = [];
                W_c = [];
                X_c = [];
                for i = 1:length(loc)
                    Y_c = [Y_c;Y_ic(t,:,i)];
                    W_c = [W_c;W_ic(t,t,i)];
                    X_c = [X_c;X_ic(t,:,i)];
                end     
                Y_betac = Y_c - X_c*gamma_draw;
                X_betac = W_c;
                er_b = Y_betac - X_betac*betac_draw(t,1);
            
                sig2vd_pst = (n_c + sig2vpr(1,1))/2; 
                sig2vnu_pst = (er_b'*er_b + sig2vpr(2,1))/2;              
                sv2_draw = 1/(gamrnd(sig2vd_pst,1/sig2vnu_pst));
                sig2m_temp(c,t) = sv2_draw;
            end
        else
            Y_betac = 0;
            X_betac = 0;    
            [betac_draw,b_pst,s_pst] = normal_bayes_reg(Y_betac,X_betac,1,beta_bpr,beta_spr); 
            betam_draw(c,:) = betac_draw';            
            
            % draw sig2v_m
            sig2vd_pst = (sig2vpr(1,1))/2; 
            sig2vnu_pst = (sig2vpr(2,1))/2;              
            sv2_draw = 1./(gamrnd(sig2vd_pst,1/sig2vnu_pst,1,T));
            sig2m_temp(c,:) = sv2_draw;            
        end

    end  
    sig2m_draw = sig2m_temp;
    
    

    % label switching moves for active set
    if Cact > 1
        if (rand(1) < 0.5) 

            %(A) randomly choose two components and propose to swap
            j = randsample(1:Cact, 2, 0);
            MHaccept = ( Nd(j(1))-Nd(j(2)) )* ( log(pi_draw(j(2))) - log(pi_draw(j(1))) );
            if log(rand(1)) < min(0,(MHaccept))
                tmp = G_draw;
                G_draw(find(tmp==j(1))) = j(2);
                G_draw(find(tmp==j(2))) = j(1);

                tmp = betam_draw;
                betam_draw(j(1),:) = tmp(j(2),:);
                betam_draw(j(2),:) = tmp(j(1),:);

                tmp = sig2m_draw;
                sig2m_draw(j(1),:) = tmp(j(2),:);
                sig2m_draw(j(2),:) = tmp(j(1),:);
            end

        else

            if (Cact > 2)
                % (B) choose a neighbouring pair to switch
                if Cact == Cmax
                    if Cact == 3
                        j = 2;
                    else
                        j = randsample(2:Cact-1, 1, 1, Nd(2:Cact-1));
                    end
                else
                    j = randsample(2:Cact, 1, 1, Nd(2:Cact));
                end

                MHaccept = Nd(j-1)*log(1-V_draw(j)) - Nd(j)*log(1-V_draw(j-1));
                if log(rand(1)) < min(0, (MHaccept))

                    tmp = V_draw;
                    V_draw(j) = tmp(j-1);
                    V_draw(j-1) = tmp(j);

                    pi_draw(:) = 0;
                    for c = 1:Cact
                        pi_draw(c,1) = V_draw(c,1)*prod(1-V_draw(1:c-1,1));
                    end

                    tmp = G_draw;
                    G_draw(find(tmp==j)) = j-1;
                    G_draw(find(tmp==j-1)) = j;

                    tmp = betam_draw;
                    betam_draw(j,:) = tmp(j-1,:);
                    betam_draw(j-1,:) = tmp(j,:);

                    tmp = sig2m_draw;
                    sig2m_draw(j,:) = tmp(j-1,:);
                    sig2m_draw(j-1,:) = tmp(j,:);
                end
            end
        end   
    end
    Cact = max(G_draw);    
    Nd = zeros(Cact,1);
    % update the number of components in each cluster    
    for c = 1:Cact
        Nd(c,1) = length( find(G_draw == c) );
    end       
    
    
    
    
    
    
    %% Step 3: draw U conditional on V
    U_draw = rand(N,1).*pi_draw(G_draw);    
 
    
    
    
    
    
    
    
    %% Step4: Draw alpha
    % simulate eta from Beta(alpha+1,n)
    eta_draw = betarnd(alpha_draw+1,N);
    
	% simulate alpha given eta
	uz = length(unique(G_draw));
	temp_wg = (uz + alpha_pr(1,1) -1)/(N*(alpha_pr(2,1)-log(eta_draw)));
	wg = temp_wg/(1+temp_wg);
	% wg is the mixture weight in the mixture of Gamma's
	alpha_draw = gamrnd(alpha_pr(1,1)+uz-randsample(2, 1, 1, [wg; 1-wg])+1, 1/(alpha_pr(2,1)-log(eta_draw)));    
          
    
    
    
    
    
    
    %% Step 5: Compute new Cmax and generate cluster-dependent parameters for inactive clusters 
    U_min = min(U_draw);
    pi_sum = sum(pi_draw);
    
    if pi_sum > (1 - U_min)
        pi_csum = cumsum(pi_draw);
        Cmax = min(find(pi_csum > (1 - U_min)));        
    else
        % sample more v's until the pi's satisfy \sum_{j=1}^{k*} pi_j > 1-min(u)
        Cmax = Cact;
        while pi_sum < (1 - U_min) 
            Cmax = Cmax + 1;
            vn_draw = betarnd(1,alpha_draw);
            pin_draw = vn_draw*prod(1-V_draw);
            V_draw = [V_draw;vn_draw];
            pi_draw = [pi_draw;pin_draw];
            pi_sum = pi_sum + pin_draw;
            
            Y_betac = 0;
            X_betac = 0; 
            [betac_draw,b_pst,s_pst] = normal_bayes_reg(Y_betac,X_betac,1,beta_bpr,beta_spr); 
            betam_draw = [betam_draw;betac_draw'];
            
            % draw sig2v_m
            sig2vd_pst = (sig2vpr(1,1))/2; 
            sig2vnu_pst = (sig2vpr(2,1))/2;              
            sv2_draw = 1./(gamrnd(sig2vd_pst,1/sig2vnu_pst,1,T));
            sig2m_draw = [sig2m_draw;sv2_draw];            
        end
    end   
    % Update for the next iteration
    index_mat = kron(G_draw,ones(T,1));
    beta_draw = betam_draw(index_mat,:);
    sig2_draw = reshape(sig2m_draw(G_draw,:)',TN,1);
    
 
    %% Step 6: draw cluster-free parameters (gamma,sigma^2) 
    % draw gamma
    tauNT_draw = 1./sqrt(sig2_draw);
    
    Y_gamma = (Y - sum(W_cat.*beta_draw,2));
    X_gamma = X;
    Y_gammas = Y_gamma.*tauNT_draw;
    X_gammas = X_gamma.*kron(ones(1,2),tauNT_draw);    
    [gamma_draw,b_pst,s_pst] = normal_bayes_reg(Y_gammas,X_gammas,1,gamma_bpr,gamma_spr); 


    
    
    
    
    
    
    
    %% Step 7: draw group indicator variables
    for i = randperm(N)
        %i = 1
        % Step 1.1: Select observations
        W_indv = W_i(:,:,i);
        X_indv = X_i(:,:,i);
        Y_indv = Y_i(:,:,i);
        U_indv = U_draw(i,1);
        
        % Step 1.2: Remove empty cluster
        Ydev = Y_indv - X_indv*gamma_draw;
        lny_den = zeros(Cmax,1);
        indy = zeros(Cmax,1);
        for c = 1:Cmax
            sig2m_mat = diag(sig2m_draw(c,:));
            lny_den(c,1) = -0.5*log(det(2*pi*sig2m_mat)) ...
                - 0.5*(Ydev - W_indv*betam_draw(c,:)')'*inv(sig2m_mat)*(Ydev - W_indv*betam_draw(c,:)');
            indy(c,1) = (U_indv < pi_draw(c,1));
        end       
        y_den = exp(lny_den - max(lny_den)).*indy;
        G_draw(i,1) = discreteRnd(y_den);
    end;
    % Update for the next iteration
    index_mat = kron(G_draw,ones(T,1));
    beta_draw = betam_draw(index_mat,:);
    sig2_draw = reshape(sig2m_draw(G_draw,:)',TN,1);
    
    
    
    
    
    %% Step 8: store MCMC samples    
    if itr > burn_in
         
        % common parameters       
        gamma_mat(itr-burn_in,:) = gamma_draw';
        G_mat(itr-burn_in,:) = G_draw';          
        alpha_mat(itr-burn_in,:) = alpha_draw;   

        % Save beta for individual countries
        beta_save = betam_draw(G_draw,:);
        beta1i_mat(itr-burn_in,:) = beta_save(:,1)';
        beta2i_mat(itr-burn_in,:) = beta_save(:,2)';
        beta3i_mat(itr-burn_in,:) = beta_save(:,3)';
        beta4i_mat(itr-burn_in,:) = beta_save(:,4)';
        beta5i_mat(itr-burn_in,:) = beta_save(:,5)';
        beta6i_mat(itr-burn_in,:) = beta_save(:,6)';
        beta7i_mat(itr-burn_in,:) = beta_save(:,7)';
        
%         sig2i_mat(itr-burn_in,:) = sig2m_draw(G_draw,:)';
        
        % Save unique beta 
        beta_unique = unique(beta_draw,'rows');
        clstn_mat(itr-burn_in,:) = size(beta_unique,1);   
        
        % compute likelihood for model comparison
        Y_gamma = (Y - sum(W_cat.*beta_draw,2));
        X_gamma = X; 
        e_draw = (Y_gamma - X_gamma*gamma_draw);
        ln_lik = log(exp(-0.5*(e_draw.^2)./sig2_draw)./sqrt(2*pi*sig2_draw));     
        lik_var = lik_var + (1/numMCMC)*((ln_lik - lik_mean(:,2)).^2);
       
    end

end
toc;

alpha_cv(:,1,conv_iter) = alpha_mat;
gnum_cv(:,1,conv_iter) = clstn_mat;
gamma_cv(:,:,conv_iter) = gamma_mat;

conv_iter = conv_iter + 1;
end


    
%% report results
% MSE Boxplot
figure (1)
boxplot([alpha_cv(:,1,1),alpha_cv(:,1,2),alpha_cv(:,1,3),alpha_cv(:,1,4),alpha_cv(:,1,5)]);
% xlabel('trial','fontsize',20)
ylabel('\beta','fontsize',20)
set(gca,'fontsize',20)
set(gca,'Xtick',1:1:5)
set(gca,'XTickLabel',{'OLS','RG1','RG2','RG3','RG4'})
% ylim([-0.6 0.2])
% legend('high(stable)','transition(early,rapid)','transition(late,rapid)','low','Location','northwest');

figure (2)
boxplot([gnum_cv(:,1,1),gnum_cv(:,1,2),gnum_cv(:,1,3),gnum_cv(:,1,4),gnum_cv(:,1,5)]);

figure (3)
boxplot([gamma_cv(:,2,1),gamma_cv(:,2,2),gamma_cv(:,2,3),gamma_cv(:,2,4),gamma_cv(:,2,5)]);
% xlabel('trial','fontsize',20)
ylabel('\theta_2','fontsize',20)
set(gca,'fontsize',20)
set(gca,'Xtick',1:1:5)
set(gca,'XTickLabel',{'OLS','RG1','RG2','RG3','RG4'})
% ylim([-0.6 0.2])
% legend('high(stable)','transition(early,rapid)','transition(late,rapid)','low','Location','northwest');


