% Kim and Le (2017): Grouped Hetrogeneity in Pennel Data Models
%
% y_{it} = x_{it}*gamma + beta_{it} + e_{it}, e_{it} ~ N(0,sigma_i^2)
% for i = 1,2,...,N, t =1,2,...,T
%
% Jaeho Kim 
% University of Oklahoma
% 2017_11_19


clear all;
clc;
close all;
rng(81);





%% Load Data
load('final_data');
[lik_mean,lik_mean_txt,lik_mean_raw] = xlsread('lik_mean.xlsx');
[coutiries_id_num,coutiries_id_txt,coutiries_id_raw] = xlsread('country_id.xls');

Z = ldem_linc;
Y = dem;
W = ones(length(Z),1);
X = Z;

T = 7;
N = 90;
K = 2;

% Contruct data matrix
Y_i = reshape(Y,T,1,N);
X_i = zeros(T,2,N);
X_i(:,1,:) = reshape(X(:,1),T,N);
X_i(:,2,:) = reshape(X(:,2),T,N);
conid_i = reshape(coutiries_id_txt,T,N);
conid_i = conid_i(1,:)';

W_i = zeros(T,T,N);
W0_i = reshape(W(:,1),T,N);
W_cat = [];

Idt_mat = eye(T);
for i = 1:N
    Wt_i = W0_i(:,i);
    for t = 1:T
        W_i(t,:,i) = kron(Idt_mat(t,:),Wt_i(t,:));
    end
    W_cat = [W_cat;W_i(:,:,i)];
end
taxis = (1970:5:2000)';






%% MCMC Estimation (saving space and priors)
[T,KW,N] = size(W_i);
TN = T*N;

numMCMC = 50000;         % Number of iterations in the MCMC samplers
burn_in = 50000;          % Number of interations to burn

% OLS Estimates
Y_ols = Y;
X_ols = [W_cat X];
ols_est = inv(X_ols'*X_ols)*X_ols'*Y_ols;
sig2_est = (Y_ols - X_ols*ols_est)'*(Y_ols - X_ols*ols_est)/TN;
ols_se = sig2_est*inv(X_ols'*X_ols);


% Prior Specification
% gamma
gamma_bpr = zeros(2,1);
gamma_spr = (10^2)*eye(2);

% beta
beta_bpr = ols_est(1:T,1);
beta_spr = 30*ols_se(1:7,1:7);


% sigma2_v
sig2vpr = 6*[1;(0.1)^2];

% Check prior for inverse gamma
% sig2vd_pst = (sig2vpr(1,1))/2; 
% sig2vnu_pst = (sig2vpr(2,1))/2;  
% sv2_draw = 1./(gamrnd(sig2vd_pst,1/sig2vnu_pst,100000,1));
% hist(sv2_draw,100)
% a = shape, b = scale
% E[x] = a*b, V[x] = a*b^2

% alpha
alpha_pr = [0.4;10];                  
           
% Check prior for gamma
% alpha_pr_test = gamrnd(alpha_pr(1,1),alpha_pr(2,1),100000,1);
% hist(alpha_pr_test,100)
% a = shape, b = scale
% E[x] = a*b, V[x] = a*b^2

% Note: when alpha is large, more likeliy to have a new cluster
%       when alpha is small, less likeliy to have a new cluster


% Saving Space
gamma_mat = zeros(numMCMC,2);
% sig2i_mat = zeros(numMCMC,N);
G_mat = zeros(numMCMC,N);
alpha_mat = zeros(numMCMC,1);
corr_mat = zeros(numMCMC,2);

clstn_mat = zeros(numMCMC,1);
beta1i_mat = zeros(numMCMC,N);
beta2i_mat = zeros(numMCMC,N);
beta3i_mat = zeros(numMCMC,N);
beta4i_mat = zeros(numMCMC,N);
beta5i_mat = zeros(numMCMC,N);
beta6i_mat = zeros(numMCMC,N);
beta7i_mat = zeros(numMCMC,N);
beta_mat = [];


% MCMC variance of log-liklihoods
lik_var = zeros(T*N,1);








%% MCMC Estimation (Initial values)
% Initial Values based on pooled OLS
gamma_draw = ols_est(8:9,1);
sig2_draw = sig2_est;
alpha_draw = 1;


% Generate candidate grouped heterogeneity from OLS estimates
Cmax = 5000;
betam_temp = (kron(ones(1,Cmax),ols_est(1:7,1)) + chol(beta_spr)'*randn(7,Cmax))';
% Draw group indicator variable
G_draw = ones(N,1);
for i = randperm(N)
    
     %i = 1
     % Step 1: Select observations
     W_indv = W_i(:,:,i);
     X_indv = X_i(:,:,i);
     Y_indv = Y_i(:,:,i);
     % Step 2: Compute density
     Ydev = Y_indv - X_indv*gamma_draw;
     lny_den = zeros(Cmax,1);
     indy = zeros(Cmax,1);
     for c = 1:Cmax
         lny_den(c,1) = -0.5*T*log(2*pi*sig2_draw) ...
             - 0.5*(Ydev - W_indv*betam_temp(c,:)')'*(Ydev - W_indv*betam_temp(c,:)')/sig2_draw;
     end       
     y_den = exp(lny_den - max(lny_den));
     G_draw(i,1) = discreteRnd(y_den);
     
 end;
% Adjust cluster index 
[G_unique,idx_a,idx_c]  = unique(G_draw);
betam_draw = betam_temp(G_unique,:);
sig2m_draw = sig2_draw*ones(length(G_unique),T);
G_draw = idx_c;
index_mat = kron(G_draw,ones(T,1));
beta_draw = betam_draw(index_mat,:);
Cmax = length(G_unique) + 100;










%% Run MCMC loop
ac_num = 0; %ratio of acceptance for nu
tic;
reverseStr = [];
for itr = 1:(numMCMC+burn_in)
    reverseStr = displayprogress(100*itr/(numMCMC+burn_in), reverseStr);
     

    
    %% Step 1: Draw V and U  (Update active components)
    % Draw V
    % count elements in clusters 
    Cact = max(G_draw);
    Nd = zeros(Cact,1);
    for c = 1:Cact
        Nd(c,1) = length( find(G_draw == c) );
    end

    % update v_1
    v_pst_a = Nd(1,1) + 1;
    v_pst_b = sum(Nd(2:end,1)) + alpha_draw;
    v1_draw = betarnd(v_pst_a,v_pst_b);

    % update v_c's upto c = Cmax and compuate weights pi_c
    % c = 1
    V_draw = zeros(Cact,1);
    pi_draw = zeros(Cact,1);
    V_draw(1,1) = v1_draw;
    pi_draw(1,1) = v1_draw;
    % c = 2,3,...,Cact-1
    for c = 2:(Cact-1)
        v_pst_a = Nd(c,1) + 1;
        v_pst_b = sum(Nd(c+1:end)) + alpha_draw;
        V_draw(c,1) = betarnd(v_pst_a,v_pst_b);
        pi_draw(c,1) = V_draw(c,1)*prod(1-V_draw(1:c-1,1));
    end
    % c = Cact
    v_pst_a = Nd(Cact,1) + 1;
    v_pst_b = alpha_draw;
    V_draw(Cact,1) = betarnd(v_pst_a,v_pst_b);
    pi_draw(Cact,1) = V_draw(Cact,1)*prod(1-V_draw(1:Cact-1,1));    
    
    
    
    
    
    
    %% Step 2: draw cluster-dependent parameters (only for active beta_c and sig2_c) 
    % For active clusters
    betam_draw = zeros(Cact,KW);
    sig2m_temp = zeros(Cact,T);
    for c = 1:Cact
        loc = find(G_draw == c);
        n_c = size(loc,1);
        if n_c > 0;
            % Find indepedent and depedent observations in a cluster
            W_ic = W_i(:,:,loc);
            Y_ic = Y_i(:,:,loc);
            X_ic = X_i(:,:,loc);
            sig2c = sig2m_draw(c,:)';
        
            % Redefine data matrix   
            Y_c = [];
            W_c = [];
            X_c = [];
            for i = 1:length(loc)
                Y_c = [Y_c;Y_ic(:,:,i)./sqrt(sig2c)];
                W_c = [W_c;diag(diag(W_ic(:,:,i))./sqrt(sig2c))];
                X_c = [X_c;X_ic(:,:,i)./sqrt([sig2c sig2c])];
            end
            
            Y_betac = Y_c - X_c*gamma_draw;
            X_betac = W_c;
            [betac_draw,b_pst,s_pst] = normal_bayes_reg(Y_betac,X_betac,1,beta_bpr,beta_spr); 
            betam_draw(c,:) = betac_draw';            
            
            % draw sig2v_m
            for t = 1:T
                Y_c = [];
                W_c = [];
                X_c = [];
                for i = 1:length(loc)
                    Y_c = [Y_c;Y_ic(t,:,i)];
                    W_c = [W_c;W_ic(t,t,i)];
                    X_c = [X_c;X_ic(t,:,i)];
                end     
                Y_betac = Y_c - X_c*gamma_draw;
                X_betac = W_c;
                er_b = Y_betac - X_betac*betac_draw(t,1);
            
                sig2vd_pst = (n_c + sig2vpr(1,1))/2; 
                sig2vnu_pst = (er_b'*er_b + sig2vpr(2,1))/2;              
                sv2_draw = 1/(gamrnd(sig2vd_pst,1/sig2vnu_pst));
                sig2m_temp(c,t) = sv2_draw;
            end
        else
            Y_betac = 0;
            X_betac = 0;    
            [betac_draw,b_pst,s_pst] = normal_bayes_reg(Y_betac,X_betac,1,beta_bpr,beta_spr); 
            betam_draw(c,:) = betac_draw';            
            
            % draw sig2v_m
            sig2vd_pst = (sig2vpr(1,1))/2; 
            sig2vnu_pst = (sig2vpr(2,1))/2;              
            sv2_draw = 1./(gamrnd(sig2vd_pst,1/sig2vnu_pst,1,T));
            sig2m_temp(c,:) = sv2_draw;            
        end

    end  
    sig2m_draw = sig2m_temp;
    
    

    % label switching moves for active set
	if (rand(1) < 0.5) 
        
		%(A) randomly choose two components and propose to swap
		j = randsample(1:Cact, 2, 0);
		MHaccept = ( Nd(j(1))-Nd(j(2)) )* ( log(pi_draw(j(2))) - log(pi_draw(j(1))) );
		if log(rand(1)) < min(0,(MHaccept))
			tmp = G_draw;
			G_draw(find(tmp==j(1))) = j(2);
			G_draw(find(tmp==j(2))) = j(1);
            
			tmp = betam_draw;
			betam_draw(j(1),:) = tmp(j(2),:);
			betam_draw(j(2),:) = tmp(j(1),:);

            tmp = sig2m_draw;
			sig2m_draw(j(1),:) = tmp(j(2),:);
			sig2m_draw(j(2),:) = tmp(j(1),:);
        end
        
    else
        
		if (Cact > 2)
			% (B) choose a neighbouring pair to switch
			if Cact == Cmax
                if Cact == 3
                    j = 2;
                else
                    j = randsample(2:Cact-1, 1, 1, Nd(2:Cact-1));
                end
			else
				j = randsample(2:Cact, 1, 1, Nd(2:Cact));
			end

			MHaccept = Nd(j-1)*log(1-V_draw(j)) - Nd(j)*log(1-V_draw(j-1));
			if log(rand(1)) < min(0, (MHaccept))

				tmp = V_draw;
				V_draw(j) = tmp(j-1);
				V_draw(j-1) = tmp(j);

				pi_draw(:) = 0;
				for c = 1:Cact
					pi_draw(c,1) = V_draw(c,1)*prod(1-V_draw(1:c-1,1));
                end
         
				tmp = G_draw;
				G_draw(find(tmp==j)) = j-1;
				G_draw(find(tmp==j-1)) = j;

				tmp = betam_draw;
				betam_draw(j,:) = tmp(j-1,:);
				betam_draw(j-1,:) = tmp(j,:);

                tmp = sig2m_draw;
				sig2m_draw(j,:) = tmp(j-1,:);
				sig2m_draw(j-1,:) = tmp(j,:);
            end
        end
    end   
    Cact = max(G_draw);    
    Nd = zeros(Cact,1);
    % update the number of components in each cluster    
    for c = 1:Cact
        Nd(c,1) = length( find(G_draw == c) );
    end       
    
    
    
    
    
    
    %% Step 3: draw U conditional on V
    U_draw = rand(N,1).*pi_draw(G_draw);    
 
    
    
    
    
    
    
    
    %% Step4: Draw alpha
    % simulate eta from Beta(alpha+1,n)
    eta_draw = betarnd(alpha_draw+1,N);
    
	% simulate alpha given eta
	uz = length(unique(G_draw));
	temp_wg = (uz + alpha_pr(1,1) -1)/(N*(alpha_pr(2,1)-log(eta_draw)));
	wg = temp_wg/(1+temp_wg);
	% wg is the mixture weight in the mixture of Gamma's
	alpha_draw = gamrnd(alpha_pr(1,1)+uz-randsample(2, 1, 1, [wg; 1-wg])+1, 1/(alpha_pr(2,1)-log(eta_draw)));    
          
    
    
    
    
    
    
    %% Step 5: Compute new Cmax and generate cluster-dependent parameters for inactive clusters 
    U_min = min(U_draw);
    pi_sum = sum(pi_draw);
    
    if pi_sum > (1 - U_min)
        pi_csum = cumsum(pi_draw);
        Cmax = min(find(pi_csum > (1 - U_min)));        
    else
        % sample more v's until the pi's satisfy \sum_{j=1}^{k*} pi_j > 1-min(u)
        Cmax = Cact;
        while pi_sum < (1 - U_min) 
            Cmax = Cmax + 1;
            vn_draw = betarnd(1,alpha_draw);
            pin_draw = vn_draw*prod(1-V_draw);
            V_draw = [V_draw;vn_draw];
            pi_draw = [pi_draw;pin_draw];
            pi_sum = pi_sum + pin_draw;
            
            Y_betac = 0;
            X_betac = 0; 
            [betac_draw,b_pst,s_pst] = normal_bayes_reg(Y_betac,X_betac,1,beta_bpr,beta_spr); 
            betam_draw = [betam_draw;betac_draw'];
            
            % draw sig2v_m
            sig2vd_pst = (sig2vpr(1,1))/2; 
            sig2vnu_pst = (sig2vpr(2,1))/2;              
            sv2_draw = 1./(gamrnd(sig2vd_pst,1/sig2vnu_pst,1,T));
            sig2m_draw = [sig2m_draw;sv2_draw];            
        end
    end   
    % Update for the next iteration
    index_mat = kron(G_draw,ones(T,1));
    beta_draw = betam_draw(index_mat,:);
    sig2_draw = reshape(sig2m_draw(G_draw,:)',TN,1);
    
 
    %% Step 6: draw cluster-free parameters (gamma,sigma^2) 
    % draw gamma
    tauNT_draw = 1./sqrt(sig2_draw);
    
    Y_gamma = (Y - sum(W_cat.*beta_draw,2));
    X_gamma = X;
    Y_gammas = Y_gamma.*tauNT_draw;
    X_gammas = X_gamma.*kron(ones(1,2),tauNT_draw);    
    [gamma_draw,b_pst,s_pst] = normal_bayes_reg(Y_gammas,X_gammas,1,gamma_bpr,gamma_spr); 


    
    
    
    
    
    
    
    %% Step 7: draw group indicator variables
    for i = randperm(N)
        %i = 1
        % Step 1.1: Select observations
        W_indv = W_i(:,:,i);
        X_indv = X_i(:,:,i);
        Y_indv = Y_i(:,:,i);
        U_indv = U_draw(i,1);
        
        % Step 1.2: Remove empty cluster
        Ydev = Y_indv - X_indv*gamma_draw;
        lny_den = zeros(Cmax,1);
        indy = zeros(Cmax,1);
        for c = 1:Cmax
            sig2m_mat = diag(sig2m_draw(c,:));
            lny_den(c,1) = -0.5*log(det(2*pi*sig2m_mat)) ...
                - 0.5*(Ydev - W_indv*betam_draw(c,:)')'*inv(sig2m_mat)*(Ydev - W_indv*betam_draw(c,:)');
            indy(c,1) = (U_indv < pi_draw(c,1));
        end       
        y_den = exp(lny_den - max(lny_den)).*indy;
        G_draw(i,1) = discreteRnd(y_den);
    end;
    % Update for the next iteration
    index_mat = kron(G_draw,ones(T,1));
    beta_draw = betam_draw(index_mat,:);
    sig2_draw = reshape(sig2m_draw(G_draw,:)',TN,1);
    
    
    
    
    
    %% Step 8: store MCMC samples    
    if itr > burn_in
         
        
        % compute correlation
        beta_corr = betam_draw(G_draw,:);
        corr_temp1 = 0;
        corr_temp2 = 0;
        for q = 1:N
           corr_temp1 = corr_temp1 + corrcoef(X_i(:,2,q),beta_corr(q,:)')/N;
        end
        for t = 1:T
           corr_temp2 = corr_temp2 + corrcoef(X_i(t,2,:),beta_corr(:,t)')/T;
        end        
        corr_mat(itr-burn_in,1) = corr_temp1(1,2);
        corr_mat(itr-burn_in,2) = corr_temp1(1,2);
        
        
        % common parameters       
        gamma_mat(itr-burn_in,:) = gamma_draw';
        G_mat(itr-burn_in,:) = G_draw';          
        alpha_mat(itr-burn_in,:) = alpha_draw;   

        % Save beta for individual countries
        beta_save = betam_draw(G_draw,:);
        beta1i_mat(itr-burn_in,:) = beta_save(:,1)';
        beta2i_mat(itr-burn_in,:) = beta_save(:,2)';
        beta3i_mat(itr-burn_in,:) = beta_save(:,3)';
        beta4i_mat(itr-burn_in,:) = beta_save(:,4)';
        beta5i_mat(itr-burn_in,:) = beta_save(:,5)';
        beta6i_mat(itr-burn_in,:) = beta_save(:,6)';
        beta7i_mat(itr-burn_in,:) = beta_save(:,7)';
        
%         sig2i_mat(itr-burn_in,:) = sig2m_draw(G_draw,:)';
        
        % Save unique beta 
        beta_unique = unique(beta_draw,'rows');
        clstn_mat(itr-burn_in,:) = size(beta_unique,1);   
        
        % compute likelihood for model comparison
        Y_gamma = (Y - sum(W_cat.*beta_draw,2));
        X_gamma = X; 
        e_draw = (Y_gamma - X_gamma*gamma_draw);
        ln_lik = log(exp(-0.5*(e_draw.^2)./sig2_draw)./sqrt(2*pi*sig2_draw));     
        lik_var = lik_var + (1/numMCMC)*((ln_lik - lik_mean(:,2)).^2);
       
    end

end
toc;
stt_itr = 1;
end_itr = numMCMC;






%% Posterior Moments
para_adj = [gamma_mat (gamma_mat(:,2)./(1-gamma_mat(:,1))) alpha_mat];
para_mean = mean(para_adj);
para_sd = std(para_adj);
para_pct = prctile(para_adj,[5 95]);

[gamma1_nelements,gamma1_centers] = hist(gamma_mat(:,1),200);
[gamma1_C,gamma1_I] = max(gamma1_nelements);
gamma1_mode = gamma1_centers(gamma1_I);

[gamma2_nelements,gamma2_centers] = hist(gamma_mat(:,2),200);
[gamma2_C,gamma2_I] = max(gamma2_nelements);
gamma2_mode = gamma2_centers(gamma2_I);

[alpha_nelements,alpha_centers] = hist(alpha_mat,200);
[alpha_C,alpha_I] = max(alpha_nelements);
alpha_mode = alpha_centers(alpha_I);




%% Posterior probabilities for cluster counts
prob_K2 = sum(clstn_mat<=2)/numMCMC;
prob_K3 = sum(clstn_mat==3)/numMCMC;
prob_K4 = sum(clstn_mat==4)/numMCMC;
prob_K5 = sum(clstn_mat==5)/numMCMC;
prob_K6 = sum(clstn_mat==6)/numMCMC;
prob_K7 = sum(clstn_mat==7)/numMCMC;
prob_K8 = sum(clstn_mat==8)/numMCMC;
prob_K9 = sum(clstn_mat==9)/numMCMC;
prob_K10 = sum(clstn_mat==10)/numMCMC;
prob_K11 = sum(clstn_mat==11)/numMCMC;
prob_K12 = sum(clstn_mat==12)/numMCMC;
prob_K13 = sum(clstn_mat==13)/numMCMC;
prob_K14 = sum(clstn_mat>14)/numMCMC;

prob_K = [prob_K2 prob_K3 prob_K4 prob_K5 prob_K6 prob_K7 prob_K8 prob_K9 prob_K10 prob_K11 prob_K12 prob_K13 prob_K14];







%% Construct Similarity Matrix
Gsave = 0;
for itr = stt_itr:numMCMC
    Gind = G_mat(itr,:);
    Gmat = [];
    for i = 1:N
        Gind_vec = zeros(N,1);
        ind1_vec = find(Gind == Gind(1,i));
        Gind_vec(ind1_vec,1) = 1;
        Gmat = [Gmat Gind_vec];
    end
    Gsave = Gsave + Gmat/(numMCMC-stt_itr);
end
figure (3)
imagesc(Gsave)
colorbar;






% Construct posterior mode of individual betas
hist_bin = 300;
beta_mode = zeros(N,8);
for i = 1:N
    
    [nelements,centers] = hist(beta1i_mat(:,i),hist_bin);
    [nmax,imax] = max(nelements);
    beta_mode(i,1) = centers(imax);
    
    [nelements,centers] = hist(beta2i_mat(:,i),hist_bin);
    [nmax,imax] = max(nelements);
    beta_mode(i,2) = centers(imax);  
    
    [nelements,centers] = hist(beta3i_mat(:,i),hist_bin);
    [nmax,imax] = max(nelements);
    beta_mode(i,3) = centers(imax);  
    
    [nelements,centers] = hist(beta4i_mat(:,i),hist_bin);
    [nmax,imax] = max(nelements);
    beta_mode(i,4) = centers(imax);  
    
    [nelements,centers] = hist(beta5i_mat(:,i),hist_bin);
    [nmax,imax] = max(nelements);
    beta_mode(i,5) = centers(imax); 
    
    [nelements,centers] = hist(beta6i_mat(:,i),hist_bin);
    [nmax,imax] = max(nelements);
    beta_mode(i,6) = centers(imax);
    
    [nelements,centers] = hist(beta7i_mat(:,i),hist_bin);
    [nmax,imax] = max(nelements);
    beta_mode(i,7) = centers(imax);    
    
end

figure (4)
plot(beta_mode(:,1:7)');
% ylim([-0.8 0.4]);








%% Post Clustering Procedure: Identify clusters from MCMC outputs
beta_mode = beta_mode(:,1:7);
[idx2,C2,sumd2] = kmeans(beta_mode,2,'start','uniform','Replicates',1000);%,'distance','cityblock');
silhouette2 = silhouette(beta_mode,idx2);%,'cityblock');
silh_measure2 = mean(silhouette2);

[idx3,C3,sumd3] = kmeans(beta_mode,3,'start','uniform','Replicates',1000);%,'distance','cityblock');
silhouette3 = silhouette(beta_mode,idx3);%,'cityblock');
silh_measure3 = mean(silhouette3);

[idx4,C4,sumd4] = kmeans(beta_mode,4,'start','uniform','Replicates',1000);%,'distance','cityblock');
silhouette4 = silhouette(beta_mode,idx4);%,'cityblock');
silh_measure4 = mean(silhouette4);

[idx5,C5,sumd5] = kmeans(beta_mode,5,'start','uniform','Replicates',1000);%,'distance','cityblock');
silhouette5 = silhouette(beta_mode,idx5);%,'cityblock');
silh_measure5 = mean(silhouette5);

[idx6,C6,sumd6] = kmeans(beta_mode,6,'start','uniform','Replicates',1000);%,'distance','cityblock');
silhouette6 = silhouette(beta_mode,idx6);%,'cityblock');
silh_measure6 = mean(silhouette6);

[idx7,C7,sumd7] = kmeans(beta_mode,7,'start','uniform','Replicates',1000);%,'distance','cityblock');
silhouette7 = silhouette(beta_mode,idx7);%,'cityblock');
silh_measure7 = mean(silhouette7);

[idx8,C8,sumd8] = kmeans(beta_mode,8,'start','uniform','Replicates',1000);%,'distance','cityblock');
silhouette8 = silhouette(beta_mode,idx8);%,'cityblock');
silh_measure8 = mean(silhouette8);

[idx9,C9,sumd9] = kmeans(beta_mode,9,'start','uniform','Replicates',1000);%,'distance','cityblock');
silhouette9 = silhouette(beta_mode,idx9);%,'cityblock');
silh_measure9 = mean(silhouette9);

[idx10,C10,sumd10] = kmeans(beta_mode,10,'start','uniform','Replicates',1000);%,'distance','cityblock');
silhouette10 = silhouette(beta_mode,idx10);%,'cityblock');
silh_measure10 = mean(silhouette10);

sumd_mat = [silh_measure2;silh_measure3;silh_measure4...
    ;silh_measure5;silh_measure6;silh_measure7;silh_measure8;silh_measure9...
    ;silh_measure10];



















%% Replication for Bonhomme and Manresa (2015)
% max(G) = 4
g1_G4 = find(idx4==1); 
g2_G4 = find(idx4==2);
g3_G4 = find(idx4==3);
g4_G4 = find(idx4==4);

beta1_g1_mat = mean(beta_mode(g1_G4,1));
beta2_g1_mat = mean(beta_mode(g1_G4,2));
beta3_g1_mat = mean(beta_mode(g1_G4,3));
beta4_g1_mat = mean(beta_mode(g1_G4,4));
beta5_g1_mat = mean(beta_mode(g1_G4,5));
beta6_g1_mat = mean(beta_mode(g1_G4,6));
beta7_g1_mat = mean(beta_mode(g1_G4,7));
beta_g1_mat = [beta1_g1_mat beta2_g1_mat beta3_g1_mat beta4_g1_mat ...
    beta5_g1_mat beta6_g1_mat beta7_g1_mat];

plot(beta_mode(g1_G4,:)')


beta1_g2_mat = mean(beta_mode(g2_G4,1));
beta2_g2_mat = mean(beta_mode(g2_G4,2));
beta3_g2_mat = mean(beta_mode(g2_G4,3));
beta4_g2_mat = mean(beta_mode(g2_G4,4));
beta5_g2_mat = mean(beta_mode(g2_G4,5));
beta6_g2_mat = mean(beta_mode(g2_G4,6));
beta7_g2_mat = mean(beta_mode(g2_G4,7));
beta_g2_mat = [beta1_g2_mat beta2_g2_mat beta3_g2_mat beta4_g2_mat ...
    beta5_g2_mat beta6_g2_mat beta7_g2_mat];

plot(beta_mode(g2_G4,:)')


beta1_g3_mat = mean(beta_mode(g3_G4,1));
beta2_g3_mat = mean(beta_mode(g3_G4,2));
beta3_g3_mat = mean(beta_mode(g3_G4,3));
beta4_g3_mat = mean(beta_mode(g3_G4,4));
beta5_g3_mat = mean(beta_mode(g3_G4,5));
beta6_g3_mat = mean(beta_mode(g3_G4,6));
beta7_g3_mat = mean(beta_mode(g3_G4,7));
beta_g3_mat = [beta1_g3_mat beta2_g3_mat beta3_g3_mat beta4_g3_mat ...
    beta5_g3_mat beta6_g3_mat beta7_g3_mat];

plot(beta_mode(g3_G4,:)')


beta1_g4_mat = mean(beta_mode(g4_G4,1));
beta2_g4_mat = mean(beta_mode(g4_G4,2));
beta3_g4_mat = mean(beta_mode(g4_G4,3));
beta4_g4_mat = mean(beta_mode(g4_G4,4));
beta5_g4_mat = mean(beta_mode(g4_G4,5));
beta6_g4_mat = mean(beta_mode(g4_G4,6));
beta7_g4_mat = mean(beta_mode(g4_G4,7));
beta_g4_mat = [beta1_g4_mat beta2_g4_mat beta3_g4_mat beta4_g4_mat ...
    beta5_g4_mat beta6_g4_mat beta7_g4_mat];

plot(beta_mode(g4_G4,:)')



% plot estimated beta's
figure (5)
subplot(1,3,1)
hold on;
plot(beta_g1_mat,'k--p','Linewidth',2);  %high k--p
plot(beta_g3_mat,'g--o','Linewidth',2);  %early
plot(beta_g4_mat,'r--d','Linewidth',2);  %late
plot(beta_g2_mat,'b--+','Linewidth',2);  %low
hold off;
xlabel('years','fontsize',20)
ylabel('time effect','fontsize',20)
set(gca,'fontsize',20)
set(gca,'Xtick',1:2:7)
set(gca,'XTickLabel',{'1970','1980','1990','2000'})
ylim([-0.5 0.3])
legend('high(stable)','transition(early,gradual)','transition(late,rapid)','low','Location','northwest');



subplot(1,3,2)
hold on;
plot(mean(Y_i(:,1,g1_G4),3),'k--p','Linewidth',2); 
plot(mean(Y_i(:,1,g3_G4),3),'g--o','Linewidth',2); 
plot(mean(Y_i(:,1,g4_G4),3),'r--d','Linewidth',2); 
plot(mean(Y_i(:,1,g2_G4),3),'b--+','Linewidth',2); 
hold off;
xlabel('years','fontsize',20)
ylabel('democracy','fontsize',20)
set(gca,'fontsize',20)
set(gca,'Xtick',1:2:7)
set(gca,'XTickLabel',{'1970','1980','1990','2000'})
ylim([0 1.45])
legend('high(stable)','transition(early,gradual)','transition(late,rapid)','low','Location','northwest');


subplot(1,3,3)
hold on;
plot(mean(X_i(:,2,g1_G4),3),'k--p','Linewidth',2); 
plot(mean(X_i(:,2,g3_G4),3),'g--o','Linewidth',2); 
plot(mean(X_i(:,2,g4_G4),3),'r--d','Linewidth',2); 
plot(mean(X_i(:,2,g2_G4),3),'b--+','Linewidth',2); 
hold off;
xlabel('years','fontsize',20)
ylabel('income','fontsize',20)
set(gca,'fontsize',20)
set(gca,'Xtick',1:2:7)
set(gca,'XTickLabel',{'1970','1980','1990','2000'})
ylim([7 11])
legend('high(stable)','transition(early,gradual)','transition(late,rapid)','low','Location','northwest');





%% Report posterior distributions (cluster-indepdent parameters)
figure(8);
subplot(1,2,1)
[fgamma,xgamma] = ksdensity(gamma_mat(stt_itr:end_itr,1));
[fgamma_pr,xgamma_pr] = ksdensity(2*randn(10000,1));
hold on;
plot(xgamma,fgamma,'k-','Linewidth',3);  
plot(xgamma_pr,fgamma_pr,'k--','Linewidth',2);  
hold off;
xlabel('\gamma_1','fontsize',40)
ylabel('density','fontsize',40)
% legend('posterior','prior','Location','northeast');
set(gca,'fontsize',40)
xlim([0 1])


[fgamma,xgamma] = ksdensity(gamma_mat(stt_itr:end_itr,2));
subplot(1,2,2)
hold on;
plot(xgamma,fgamma,'k-','Linewidth',3);  
plot(xgamma_pr,fgamma_pr,'k--','Linewidth',2);  
hold off;
xlabel('\gamma_2','fontsize',40)
ylabel('density','fontsize',40)
% legend('posterior','prior','Location','northeast');
set(gca,'fontsize',40)
xlim([0 0.15])



filename = 'gamma_model4.xlsx';
xlswrite(filename,gamma_mat)










%% Gouped data path for democracy and GDP
figure (10)
subplot(4,2,1);
hold on;
plot(reshape(Y_i(:,1,g1_G4),T,length(g1_G4)));
plot(mean(Y_i(:,1,g1_G4),3)-std(Y_i(:,1,g1_G4),0,3),'k--','Linewidth',2.5);
plot(mean(Y_i(:,1,g1_G4),3),'k-','Linewidth',3);
plot(mean(Y_i(:,1,g1_G4),3)+std(Y_i(:,1,g1_G4),0,3),'k--','Linewidth',2.5);
hold off;
ylim([-.1 1.1])

subplot(4,2,2);
hold on;
plot(reshape(X_i(:,2,g1_G4),T,length(g1_G4)));
plot(mean(X_i(:,2,g1_G4),3)-std(X_i(:,2,g1_G4),0,3),'k--','Linewidth',2.5);
plot(mean(X_i(:,2,g1_G4),3),'k-','Linewidth',3);
plot(mean(X_i(:,2,g1_G4),3)+std(X_i(:,2,g1_G4),0,3),'k--','Linewidth',2.5);
hold off;
ylim([6 10.5])

subplot(4,2,3);
hold on;
plot(mean(Y_i(:,1,g2_G4),3)-std(Y_i(:,1,g2_G4),0,3),'k--','Linewidth',2.5);
plot(mean(Y_i(:,1,g2_G4),3),'k-','Linewidth',3);
plot(mean(Y_i(:,1,g2_G4),3)+std(Y_i(:,1,g2_G4),0,3),'k--','Linewidth',2.5);
plot(reshape(Y_i(:,1,g2_G4),T,length(g2_G4)));
hold off;
ylim([-.1 1.1])

subplot(4,2,4);
hold on;
plot(reshape(X_i(:,2,g2_G4),T,length(g2_G4)));
plot(mean(X_i(:,2,g2_G4),3)-std(X_i(:,2,g2_G4),0,3),'k--','Linewidth',2.5);
plot(mean(X_i(:,2,g2_G4),3),'k-','Linewidth',3);
plot(mean(X_i(:,2,g2_G4),3)+std(X_i(:,2,g2_G4),0,3),'k--','Linewidth',2.5);
hold off;
ylim([6 10.5])


subplot(4,2,5);
hold on;
plot(mean(Y_i(:,1,g3_G4),3)-std(Y_i(:,1,g3_G4),0,3),'k--','Linewidth',3);
plot(mean(Y_i(:,1,g3_G4),3),'k-','Linewidth',3);
plot(mean(Y_i(:,1,g3_G4),3)+std(Y_i(:,1,g3_G4),0,3),'k--','Linewidth',3);
plot(reshape(Y_i(:,1,g3_G4),T,length(g3_G4)));
hold off;
ylim([-.1 1.1])

subplot(4,2,6);
hold on;
plot(reshape(X_i(:,2,g3_G4),T,length(g3_G4)));
plot(mean(X_i(:,2,g3_G4),3)-std(X_i(:,2,g3_G4),0,3),'k--','Linewidth',2.5);
plot(mean(X_i(:,2,g3_G4),3),'k-','Linewidth',3);
plot(mean(X_i(:,2,g3_G4),3)+std(X_i(:,2,g3_G4),0,3),'k--','Linewidth',2.5);
hold off;
ylim([6 10.5])


subplot(4,2,7);
hold on;
plot(mean(Y_i(:,1,g4_G4),3)-std(Y_i(:,1,g4_G4),0,3),'k--','Linewidth',2.5);
plot(mean(Y_i(:,1,g4_G4),3),'k-','Linewidth',3);
plot(mean(Y_i(:,1,g4_G4),3)+std(Y_i(:,1,g4_G4),0,3),'k--','Linewidth',2.5);
plot(reshape(Y_i(:,1,g4_G4),T,length(g4_G4)));
hold off;
ylim([-.1 1.1])


subplot(4,2,8);
hold on;
plot(reshape(X_i(:,2,g4_G4),T,length(g4_G4)));
plot(mean(X_i(:,2,g4_G4),3)-std(X_i(:,2,g4_G4),0,3),'k--','Linewidth',2.5);
plot(mean(X_i(:,2,g4_G4),3),'k-','Linewidth',3);
plot(mean(X_i(:,2,g4_G4),3)+std(X_i(:,2,g4_G4),0,3),'k--','Linewidth',2.5);
hold off;
ylim([6 10.5])



%% WAIC for model comparison
WAIC = - sum(log(lik_mean(:,1))) + sum(lik_var);





%%Probability of group membership
country_id = [];
for i = 1:N
    country_id = [country_id;coutiries_id_raw(i*T,1)];
end

contry_benchmark = zeros(4,1);
beta_gmean = [beta_g1_mat;beta_g2_mat;beta_g3_mat;beta_g4_mat];
for g = 1:4
    
    if g == 1
        g_ind = g1_G4;
    elseif g == 2
        g_ind = g2_G4;        
    elseif g == 3
        g_ind = g3_G4; 
    elseif g == 4
        g_ind = g4_G4;                
    end
    
    % Count the number of zeros in Gsave
    zero_count = zeros(length(g_ind),1);
    for i = 1:length(g_ind)
        zero_count(i,1) = sum(Gsave(g_ind(i,1),:) == 0);
    end
    [max_val,max_ind] = max(zero_count);
    contry_benchmark(g,1) = g_ind(max_ind(1,1),1);
end



% switch label
country_id(contry_benchmark)
figure (11)
hold on
plot(1:T,Y_i(:,:,contry_benchmark(1)),'k-')
plot(1:T,Y_i(:,:,contry_benchmark(2)),'r-')
plot(1:T,Y_i(:,:,contry_benchmark(3)),'b-')
plot(1:T,Y_i(:,:,contry_benchmark(4)),'g-')
hold off
legend('1','2','3','4');

contry_benchmark_temp = zeros(4,1);
contry_benchmark_temp(1) = contry_benchmark(1);
contry_benchmark_temp(2) = contry_benchmark(3);
contry_benchmark_temp(3) = contry_benchmark(4);
contry_benchmark_temp(4) = contry_benchmark(2);

contry_benchmark = contry_benchmark_temp;
figure (12)
hold on
plot(1:T,Y_i(:,:,contry_benchmark(1)),'k-')
plot(1:T,Y_i(:,:,contry_benchmark(2)),'r-')
plot(1:T,Y_i(:,:,contry_benchmark(3)),'b-')
plot(1:T,Y_i(:,:,contry_benchmark(4)),'g-')
hold off
legend('1','2','3','4');



% compuate the probability of group membership
gprob_mat = zeros(N,4);
for i = 1:N
    for c = 1:4
        c_b = contry_benchmark(c,1);
        % benchmark country
        gprob_mat(i,c) =  Gsave(i,c_b);
    end
end

gprob_final = [gprob_mat(:,1) gprob_mat(:,2) gprob_mat(:,3) gprob_mat(:,4)];

% report result
rowLabels = country_id;
columnLabels = {'Group 1', 'Group 2', 'Group 3', 'Group 4'};
matrix2latex(gprob_final, 'group_prob.tex', 'rowLabels', rowLabels, 'columnLabels', columnLabels, 'alignment', 'c', 'format', '%-6.3f', 'size', 'tiny');

filename = 'groupprob_model4.xlsx';
xlswrite(filename,gprob_final)

filename = 'contry_benchmark.xlsx';
xlswrite(filename,contry_benchmark)


% correlation
figure (13)
[fcorr,xcorr] = ksdensity(corr_mat(:,2));
plot(xcorr,fcorr,'k-','Linewidth',3);  
xlabel('\gamma_1','fontsize',40)
ylabel('density','fontsize',40)
% legend('posterior','prior','Location','northeast');
set(gca,'fontsize',40)


